;; Generated package description from shell-quasiquote.el  -*- mode: lisp-data; no-byte-compile: t -*-
(define-package "shell-quasiquote" "0.0.20250316.162215" "Turn s-expressions into shell command strings." 'nil :commit "cc170c2f48c9e286e38a8afc14007ccdc46955ed" :url "https://elpa.gnu.org/packages/shell-quasiquote.html" :authors '(("Taylan Ulrich Bayırlı/Kammer" . "taylanbayirli@gmail.com")) :maintainer '("Taylan Ulrich Bayırlı/Kammer" . "taylanbayirli@gmail.com") :keywords '("extensions" "unix"))
